package io.irandom.decision;

import java.util.Vector;
import salsa.language.Actor;
import salsa.language.ServiceFactory;
import io.core.decision.DecisionAgent;
import io.core.language.IOActor;
import io.core.messaging.IOTheater;
import io.core.decision.DecisionAgent;
import io.random.decision.RandomDecisionAgent;
import io.irandom.language.ThroughputActor;
import io.irandom.protocol.IRandomSteal;
import io.irandom.messaging.IRandomPeerTheater;

public class IRandomDecisionAgent extends RandomDecisionAgent implements DecisionAgent {

	//Waiting determines if the decision agent will originate another random stealing packet.
	private boolean waiting = false;
	private IOTheater theater;
	
	private long lastReset;
	private double bandwidth;
	private double processing;

	public IRandomDecisionAgent(double _bandwidth, double _processing) {
		theater = (IOTheater)ServiceFactory.getTheater();
		
		lastReset = System.currentTimeMillis();

		bandwidth = _bandwidth;
		processing = _processing;
	}

	private double decisionFunction(ThroughputActor actor, IRandomSteal stealPacket) {
		double foreignProcessed = 0;

		if (actor.getProcessed() > 0) {
			//Find the average cycles per message processed
			double cycles = actor.getProcessingTime() * processing / actor.getProcessed();

			//Find the time it will take to process single messages on the foreign theater
			double foreignProcessTime = cycles / stealPacket.foreignUnusedProcessing;

			double foreignTotalProcessTime = 0;

			if (actor.getSentForeign() > 0) {
				//Find the average bits per message sent to a foreign theater
				double bits = (actor.getSendingTime() * bandwidth) / actor.getSentForeign();

				//Find the time it will take to send single messages on the foreign theater
				double foreignSendTime = bits / stealPacket.foreignUnusedBandwidth;

				//To fully process one message (including sending what it sends) on foreign theater
				double sendingRatio = actor.messagesNotSentTo(stealPacket.getOrigin()) / actor.getProcessed();

				foreignTotalProcessTime = (foreignProcessTime + (sendingRatio * foreignSendTime));
			} else {
				foreignTotalProcessTime = foreignProcessTime;
			}

			foreignProcessed = actor.getCollectionTime() / foreignTotalProcessTime;
		} else {
			foreignProcessed = actor.getRecieved();
		}

		return foreignProcessed - actor.getProcessed();
	}

	public IOActor getBestCandidate(IRandomSteal stealPacket) {
		//returns the best candidate actor to migrate to the given theater

		Vector registry = new Vector(theater.getRegistry().values());

		IOActor candidate = null;
		double candidateGain = -1;

		for (int i = 0; i < registry.size(); i++) {
			if (registry.get(i) instanceof ThroughputActor) {
				ThroughputActor current = (ThroughputActor)registry.get(i);

				if (!current.scheduled()) {
					double gain = decisionFunction(current, stealPacket);

					if ((!current.satisfied()) && (gain > candidateGain)) {
						candidate = (IOActor)current;
						candidateGain = gain;
					}
				}
			}
		}

		return candidate;
	}

	private void beginSteal() {
		if (!theater.getNeighbors().isEmpty()) {
			System.err.println("Sending random steal packet");
			waiting = true;

			//create a RandomSteal packet, and send it to a random neighbor

			Vector registry = new Vector(theater.getRegistry().values());
	
			long sendingUsed = 0;
			long processingUsed = 0;
			double unusedProcessing = 0;
			double unusedBandwidth = 0;
	
			for (int i = 0; i < registry.size(); i++) {
				if (registry.get(i) instanceof ThroughputActor) {
					sendingUsed += ((ThroughputActor)registry.get(i)).getSendingTime();
					processingUsed += ((ThroughputActor)registry.get(i)).getProcessingTime();
				}
			}

			long collectionTime = System.currentTimeMillis() - lastReset;
	
			unusedBandwidth  = 1 - ((bandwidth * sendingUsed)/collectionTime);
			unusedProcessing = 1 - ((processing * processingUsed)/collectionTime);

			IRandomSteal iRandomSteal = new IRandomSteal(theater.getLocation(), unusedProcessing, unusedBandwidth);
			iRandomSteal.randomSend(theater.getNeighbors());
		}
	}
}
