/**
	Author: Travis Desell
	Last Modified: 9/19/2002
*/

package io.irandom.language;

import java.io.*;
import java.net.*;
import java.util.Hashtable;
import salsa.naming.*;
import salsa.naming.exceptions.*;
import salsa.language.*;
import salsa.language.exceptions.*;
import io.core.language.IOActor;
import io.core.messaging.TheaterLocator;
import io.core.messaging.IOTheater;

public class ThroughputActor extends Actor implements IOActor {
	protected int messagesRecieved = 0;
	protected int messagesSent = 0;
	protected int messagesSentForeign = 0;
	protected int messagesProcessed = 0;
	protected boolean scheduled = false;

	protected long sendingTime = 0;
	protected long processingTime = 0;
	protected long lastResetTime = 0;

	private Hashtable targetList;

	public ThroughputActor() {
		super();
		reset();
	}

	public void reset() {
		messagesProcessed = 0;
		messagesSent = 0;
		messagesSentForeign = 0;
		messagesRecieved = 0;

		lastResetTime = System.currentTimeMillis();
		processingTime = 0;
		sendingTime = 0;

		targetList = new Hashtable();
	}

	public boolean scheduled() {
		return scheduled;
	}

	public int getRecieved() {
		return messagesRecieved;
	}

	public int getProcessed() {
		return messagesProcessed;
	}

	public long getCollectionTime() {
		return System.currentTimeMillis() - lastResetTime;
	}

	public long getProcessingTime() {
		return processingTime;
	}

	public long getSendingTime() {
		return sendingTime;
	}

	public long getSentForeign() {
		return sendingTime;
	}

	public void incrementSent() {
		messagesSent++;
	}

	public void incrementForeignSend(String targetLocation, long sendTime) {
		sendingTime += sendTime;
		messagesSentForeign++;

		/* add the target location to our list of peers and message sent to them */
		Object sentTo = targetList.get(targetLocation);
		if (sentTo != null) {
			sentTo = new Integer( ((Integer)sentTo).intValue() + 1 );
		} else {
			targetList.put(targetLocation, new Integer(1));
		}

		System.out.println("targetList.get( " + targetLocation + " ) = " + (Integer)targetList.get(targetLocation));
	}

	public int messagesNotSentTo(TheaterLocator target) {
		Object sentTo = targetList.get(target.toString());
		if (sentTo == null) return messagesSent;
		else return messagesSent - ((Integer)sentTo).intValue();
	}

        public void prioritySend(Message message) {
                scheduled = true;
		messagesRecieved++;
                super.prioritySend(message);
	}

        public void send(Message message) {
                messagesRecieved++;
                super.send(message);
        }

	public void migrate(String ualString) {
		//set scheduled to false so when it reaches the other theater
		//it may migrate from there
		reset();
		scheduled = false;

		super.migrate(ualString);
	}

	public synchronized void process(Message message) {
		long beforeProcessTime = System.currentTimeMillis();

		super.process(message);

		processingTime += System.currentTimeMillis() - beforeProcessTime;

		messagesProcessed++;
	}

	public boolean satisfied() {
		//we subtract one from messages recieved in case the actor is processing
		//messages as quickly as it is recieving them, and is in the middle of processing
		//a message
                        
		if (mailbox.isEmpty()) {
			if (messagesProcessed == 0) return true;
			else if (messagesProcessed > messagesRecieved) return true;
			else return false;
		} else {
			if (messagesProcessed > messagesRecieved) {
				return true;
			} else {
				return false;
			}
		}
        }

        public double satisfaction() {
                if (messagesProcessed > 0) {
                        return (messagesRecieved - messagesProcessed) / messagesProcessed;
                } else {
                        return (messagesRecieved * 2);
                }
        }
}
