/**
	Autonomous Actor:
	Autonmous Actor contains the data gathering capabilities for an individual
	Actor to learn about its surroundings (in terms of processing speed and
	message sending speed).  The Peer Theater uses this information to determine
	when and if to migrate Autonomous Actors around IO.

	Author: Travis Desell
	Last Modified: 9/20/2002
*/

package io.irandom.language;

import salsa.language.ActorReference;
import salsa.language.Message;
import salsa.language.ServiceFactory;
import salsa.naming.UAN;
import salsa.naming.UAL;
import salsa.messaging.TransportService;
import salsa.messaging.TheaterService;

import io.core.messaging.TheaterLocator;

public class ThroughputActorReference extends ActorReference {
	private TransportService transportService = ServiceFactory.getTransport();
	private TheaterService theaterService = ServiceFactory.getTheater();

	public ThroughputActorReference() {
		super();
	}

	public ThroughputActorReference(UAN uan) {
		super(uan);
	}

	public ThroughputActorReference(UAL ual) {
		super(ual);
	}

	/*
	 * Replaces the normal actors send to provide some profiling information.
	 */
	public void send(Message message) {
		//If this actor has been sent a bind message, its UAN needs to be updated
		//so it can be serialized and access its state on the remote theater,  move the
		//states registry entry to the non-anonymous registry, and update the
		//anonymous registry with the uan

		message.messageSent = System.currentTimeMillis();

		//if the actor is local, get its state and put the message
		//in its mailbox.
		if (getActor() != null) {
			synchronized (getActor().mailbox) {
				//local message send
				if (getActor() != null) {
					getActor().send(message);

					if (message.getSource().getActor() != null) {
						((ThroughputActor)message.getSource().getActor()).incrementSent();
					}

					return;
				}
			}
		}

		//remote message send:   
		//if the actor is non-anonymous(it has a UAN), get its UAL from the registry
		//if not, remote send the message to the specified UAL

		long beforeSendTime = System.currentTimeMillis();

		if (uan == null) transportService.send(message, ual);
		else {   
			UAL remoteLocation = (UAL)theaterService.getEntry(uan);

			transportService.send(message, remoteLocation);
		}

		if (message.getSource().getActor() != null) {
			String targetLocation = message.getTarget().getUAL().getLocation().toString();
			((ThroughputActor)message.getSource().getActor()).incrementForeignSend(targetLocation, System.currentTimeMillis() - beforeSendTime);
		}
	}
}
