/** SALSA/World Wide Computer Project
 *  ObjectHandler recieves incoming objects
 *  and determines if they are actors or messages.
 *  upon recieving actors, it will start them up.
 *  messages get sent to the appropriate actor.
 *
 *  author Travis Desell
 */
package io.random.messaging;

import java.net.*;
import java.io.*;
import java.util.Vector;
import java.util.Random;
import salsa.language.*;
import salsa.naming.*;
import io.core.language.*;
import io.core.protocol.P2PPacket;
import io.core.messaging.ProtocolHandler;
import io.core.messaging.IOTheater;
import io.core.messaging.TheaterLocator;
import io.core.decision.DecisionAgent;
import io.random.language.RandomActorReference;
import io.random.protocol.*;
import io.random.decision.RandomDecisionAgent;
	
public class RandomProtocolHandler implements ProtocolHandler {
	public IOTheater theater;

	int maxTimeToLive = 10;

	public RandomProtocolHandler() {
		theater = (IOTheater)ServiceFactory.getTheater();
	}

	public void process(P2PPacket recieved) {
		RandomDecisionAgent decider = (RandomDecisionAgent)theater.getDecisionAgent();

		if (recieved instanceof RandomSteal) {
			RandomSteal randomSteal = (RandomSteal)recieved;

			IOActor candidate = decider.getBestCandidate();

			if ( candidate != null ) {
				//if we should migrate an actor to the origin of this RandomSteal packet,
				//priority send that actor a message to do so.

				String migrationTarget = randomSteal.getOrigin().toString() + ((Actor)candidate).getUAN().getIdentifier();

				ActorReference[] _targets = { new RandomActorReference(candidate.getUAN()) };
				String[] _methodNames = { "migrate" };
				Object[][] _arguments = { {migrationTarget} };
                
				candidate.prioritySend( new Message( null, _targets, _methodNames, _arguments ) );
				decider.reset();
			} else {
				if (randomSteal.timeToLive > maxTimeToLive) {
					randomSteal.timeToLive = maxTimeToLive;
				}
				randomSteal.timeToLive--;

				if ((randomSteal.timeToLive > 0) && (canSend(randomSteal.getOrigin(), theater.getNeighbors()))) {
					//send the random steal packet to a random neighbor that is not the originator
					randomSteal.randomSend(theater.getNeighbors());
				} else {
					//tell the originating theater that its RandomSteal packet has been terminated
					Terminated terminated = new Terminated(theater.getLocation());
					terminated.send(randomSteal.getOrigin());							
				}
			}
		} else if (recieved instanceof Terminated) {
			decider.stealTerminated();
		}


		//add the originator of this packet to our list of peers,
		//if its not already there
		addPeer( recieved.getOrigin() );
   	}

	private void addPeer(TheaterLocator peer) {
		Vector neighbors = theater.getNeighbors();

		for (int i = 0; i < neighbors.size(); i++) {
			if (peer.equals( (TheaterLocator)neighbors.get(i) )) return;
		}

		theater.getNeighbors().add(peer);
	}

	private boolean canSend(TheaterLocator packetOrigin, Vector targets) {
		if (targets.size() < 1) return false;
		else if ((targets.size() == 1) && (targets.contains(packetOrigin))) return false;
		else return true;
	}
}

