/**
 * The Mailbox class holds an actor's mailbox.  
 */

package salsa.language;

import java.util.Vector;

public class Mailbox implements java.io.Serializable{
	Actor actor;
	Vector messages;

	protected Mailbox(Actor actor) {
		this.actor = actor; 
		messages = new Vector();
	}

	protected int size() {
		return messages.size();
	}

	protected synchronized void putFront(Message message) {
		//put a priority message in front of the mailbox
		
		message.messageQueued = System.currentTimeMillis();
		
		messages.insertElementAt(message, 0);
	}

	protected synchronized void put(Message message){
		
		message.messageQueued = System.currentTimeMillis();
		
		messages.addElement(message);
		notify();            // Mailbox is not empty anymore!
	}

	protected synchronized Message get(){
		if (isEmpty()) {
	    		try {
				wait();      // The lock of Mailbox means it is empty.
			} catch (InterruptedException ie){
				System.err.println("Getting a new message:"+ie);
	    		}
		}
		Message m = (Message) messages.firstElement();
		messages.removeElementAt(0);
		if (isEmpty()) {
			actor.notifyEmptyMailbox();  // Mailbox has become empty!
		}
		return m;
	}

	public boolean isEmpty(){
		return messages.isEmpty();
	}
}
