/** SALSA/World Wide Computer Project
 *  ObjectHandler recieves incoming objects
 *  and determines if they are actors or messages.
 *  upon recieving actors, it will start them up.
 *  messages get sent to the appropriate actor.
 *
 *  author Travis Desell
 */
package wwc.messaging;

import java.net.*;
import java.io.*;
import salsa.language.Actor;
import salsa.language.Message;
import salsa.language.RunTime;
import salsa.language.ServiceFactory;
import salsa.messaging.*;

public class SocketHandler implements ReceptionService {
	public SocketHandler() {
	}

        public void processActor(Actor incomingState, Socket incoming) {
                //add the state to the stateRegistry, so incoming actor references can
                //get to it
                ServiceFactory.getTheater().setEntry(incomingState.getUAN(), incomingState);

                RunTime.receivedUniversalActor();

                // Send an RmspAcknowledgement.
                try {
                        new ObjectOutputStream( incoming.getOutputStream() ).writeObject( new RMSPAcknowledgement() );
                } catch (IOException e) {
                        System.err.println( "Could not send RmspAcknowledgement is the remote theater compliant? " + e.getMessage() );
                }
        }

        public void processMessage(Message m) {
                // Send the message.
                m.getTarget().send(m);
        }

	public void process(Socket incoming) {
		// Read in the object.
		Object received = null;
		ObjectInputStream stream = null;
		try {
			stream = new ObjectInputStream( incoming.getInputStream() );
			received = stream.readObject();
		} catch (ClassNotFoundException e) {
			System.err.println( "Could not load class for " + received.getClass().getName() );
			System.err.println( "  Is it in the theater CLASSPATH" );
			System.err.println( e.getMessage() );
			return;
		} catch (IOException e) {
			System.err.println( "IOException occured reading object " + e.getMessage() );
			return;
		}

		if (received instanceof Actor) {
			Actor incomingActor = (Actor) received;
			processActor(incomingActor, incoming);
		} else if (received instanceof Message) {  // just serializing data
			Message message = (Message) received;
			processMessage(message);
		}

		// Close the connection.
		try {
			incoming.close();
		} catch (IOException e) {
			System.err.println( "IOException occured closing connection" + e.getMessage() );
			return;
		}
	}
}
